/*
    This program demonstrates a very subtle race condition.

    If you run this program on a multicore machine, then this
    program will demonstrate a race condition.

    If you compile this program in the usual way with either
    cl, gcc, or lcc, and then run this program on a single
    core machine (or restrict the affinity of this program
    to a single core on a multicore machine), then this
    program will NOT have a race condition.

    Moreover, if you compile this program using gcc and the
    compiler option -fforce-mem, and then run this program on
    a single core machine (or restrict the affinity of this
    program to a single core on a multicore machine), then
    you do get a race condition.

    The compiler option -fforce-mem tells gcc to compile in a
    way that mimics RISC type machines. So the assembly language
    code for count++ changes from the usual i386 code with an
    atomic memory access
      incl  _count
    to the more RISC like
      movl  _count, %eax
      incl  %eax
      movl  %eax, _count
    The extra instructions allow for a race condition between the
    threads on a single core (where does the race condition come
    from with the "atomic" incl instruction on a multicore machine?).

    This program shows that code can have a race condition that only
    shows up when the code is compiled for certain kinds of CPU's.

    This example is based on Figure 13.16 badcnt.c, page 872, from
    "Computer Systems: A Programmers Perspective",
    by R Bryant and D. O'Hallaron.
*/
#include <windows.h>
#include <stdio.h>

#define ITERATIONS 100000000 // 100,000,000

// global shared variable
unsigned count = 0;

int main( int argc, char *argv[])
{
   DWORD WINAPI thread(LPVOID);  //thread function prototype

   #define NUMBER_OF_THREADS 6
   HANDLE hthread[NUMBER_OF_THREADS];
   int threadParam = 0;  // thread function parameter
   int i;

   // create threads
   printf("Starting %d threads\n", NUMBER_OF_THREADS);
   for (i = 0; i < NUMBER_OF_THREADS; i++)
      hthread[i] = CreateThread(NULL, 0, thread, &threadParam, 0, NULL);

   // wait for the threads to finish
   WaitForMultipleObjects(NUMBER_OF_THREADS, hthread, TRUE, INFINITE);

   printf("\nThreads stopped.\n");

   if (count != (unsigned)ITERATIONS*NUMBER_OF_THREADS)
      printf("A RACE! count=%d (not %d)\n", count, (unsigned)ITERATIONS*NUMBER_OF_THREADS);
   else
      printf("No race: count=%d\n", count);

   return 0;
}//main



DWORD WINAPI thread(LPVOID threadParam)
{
   int i;
   for (i = 0; i < ITERATIONS; i++)
      count++;

   return 0;
}//thread
