/*
   This program shows that a Win32 mutex remembers how many
   times its owning thread waited on it. The owning thread
   must release the mutex once for every time that the thread
   waited on the mutex.

   Also, if a thread terminates while owing a mutex, it seems that
   the mutex will then be released.
*/
#include <windows.h>
#include <stdio.h>

HANDLE mutex;

int main( int argc, char *argv[])
{
   DWORD WINAPI thread1(LPVOID);  //thread function prototypes
   DWORD WINAPI thread2(LPVOID);

   int threadParam1 = 0;  // thread function parameter
   int threadParam2 = 0;

   // create a Mutex object (in signaled state)
   mutex = CreateMutex(NULL, FALSE, NULL);

   // create threads
   printf("Starting threads\n");
   CreateThread(NULL, 0, thread1, &threadParam1, 0, NULL);
   CreateThread(NULL, 0, thread2, &threadParam2, 0, NULL);

   Sleep(15000);
   printf("\nThreads stopped.\n");

   return 0;
}//main


DWORD WINAPI thread1(LPVOID threadParam)
{
   WaitForSingleObject(mutex, INFINITE); // neither of these waits will block
   WaitForSingleObject(mutex, INFINITE);
   printf("thread1 now has the mutex.\n");
   Sleep(5000);
   printf("thread1 is releasing the mutex?\n");
   ReleaseMutex(mutex);
   Sleep(5000);
   printf("thread1 is releasing the mutex again.\n");
   ReleaseMutex(mutex);
   Sleep(4000);

   printf("thread1 is done.\n");
   return 0;
}//thread1


DWORD WINAPI thread2(LPVOID threadParam)
{
   Sleep(4000);
   printf("thread2 wants the mutex.\n");
   WaitForSingleObject(mutex, INFINITE);
   printf("thread2 now has the mutex.\n");
   Sleep(2000);
   printf("thread2 is done.\n");
   return 0;
}//thread2
