/*
    This program uses a mutex to avoid the race condition.

    This program runs one of the threads at a higher priority
    than the other thread. The higher priority thread nearly
    monopolizes the critical section. This shows that thread
    synchronization can have (complicated) interactions with
    other features of the operating system.
*/
#include <windows.h>
#include <stdio.h>

// global variables
int runFlag = TRUE;
unsigned int count  = 0;  // this variable is shared by thread1 and thread2
unsigned int count1 = 0;  // this variable is used only by thread1
unsigned int count2 = 0;  // this variable is used only by thread2

HANDLE mutex;

int main( int argc, char *argv[])
{
   DWORD WINAPI thread1(LPVOID);  //thread function prototypes
   DWORD WINAPI thread2(LPVOID);

   HANDLE thread1Handle;
   HANDLE thread2Handle;
   int threadParam1 = 0;  // thread function parameter
   int threadParam2 = 0;

   // create a Mutex object (in signaled state)
   mutex = CreateMutex(NULL, FALSE, NULL);

   // create the threads
   thread1Handle = CreateThread(NULL, 0, thread1, &threadParam1, CREATE_SUSPENDED, NULL);
   thread2Handle = CreateThread(NULL, 0, thread2, &threadParam2, CREATE_SUSPENDED, NULL);

   // set the priority of the (suspended) threads
   SetThreadPriority(thread1Handle, THREAD_PRIORITY_ABOVE_NORMAL);
   SetThreadPriority(thread2Handle, THREAD_PRIORITY_BELOW_NORMAL);

   // now start the threads
   printf("race3m: Starting threads\n");
   ResumeThread(thread1Handle);
   ResumeThread(thread2Handle);

   // let the threads run for a while
   Sleep(2000);
   // now stop the threads
   runFlag = FALSE;
   Sleep(20);

   printf("\nThreads stopped.\n");
   printf("count = %d, count1 = %d, count2 = %d\n", count, count1, count2);

   return 0;
}//main



DWORD WINAPI thread1(LPVOID threadParam)
{
   unsigned int new_count, old_count;
   while(runFlag)
   {
      WaitForSingleObject(mutex, INFINITE);
      old_count = count++;
      new_count = count;
      ReleaseMutex(mutex);

      if (new_count - old_count != 1)
         printf("Bingo1: new_count - old_count = %d\n", new_count-old_count);
      count1++;
   }
   return 0;
}//thread1


DWORD WINAPI thread2(LPVOID threadParam)
{
   unsigned int new_count, old_count;
   while(runFlag)
   {
      WaitForSingleObject(mutex, INFINITE);
      old_count = count++;
      new_count = count;
      ReleaseMutex(mutex);

      if (new_count - old_count != 1)
         printf("Bingo2: new_count - old_count = %d\n", new_count-old_count);
      count2++;
   }
   return 0;
}//thread2
