/*
    This program demonstrates how a simple "race condition" can come
    up when threads share a variable.

    The shared variable is the loop index variable i. Since each
    thread is passed a pointer to i, each thread is accessing the
    same memory location for its thread parameter. The actual value
    that any thread sees in its thread parameter depends on when the
    thread is started. That is, the actual value that a thread sees
    for its thread parameter will be the value in i when the thread
    starts, NOT the value that was in i when CreateThread() was called
    to create the thread.
*/
#include <windows.h>
#include <stdio.h>

int main( int argc, char *argv[])
{
   DWORD WINAPI thread(LPVOID);  //thread function prototype

   #define NUMBER_OF_THREADS 10
   HANDLE hthread[NUMBER_OF_THREADS];
   int i;

   // create threads
   printf("Starting %d threads\n", NUMBER_OF_THREADS);
   for (i = 0; i < NUMBER_OF_THREADS; i++)
   {
      hthread[i] = CreateThread(NULL, 0, thread, &i, 0, NULL);
   }

   // wait for the threads to finish
   WaitForMultipleObjects(NUMBER_OF_THREADS, hthread, TRUE, INFINITE);

   printf("\nThreads stopped.\n");

   return 0;
}//main


DWORD WINAPI thread(LPVOID threadParam)
{
   int myParameter = *((int *)threadParam);
   printf("Hello from thread %d\n", myParameter);

   return 0;
}//thread
