/*
    This program demonstrates how a simple race condition can be
    removed.

    This program removes the race condition by removing the shared
    variable. In this case, instead of passing each thread a pointer
    to the index variable, the thread is passed a pointer to a
    dynamically created copy of the index variable. So each thread
    gets its own dynamically allocated thread param
*/
#include <windows.h>
#include <stdio.h>

int main( int argc, char *argv[])
{
   DWORD WINAPI thread(LPVOID);  //thread function prototype

   #define NUMBER_OF_THREADS 10
   HANDLE hthread[NUMBER_OF_THREADS];
   int i;
   void* ptr;

   // create threads
   printf("Starting %d threads\n", NUMBER_OF_THREADS);
   for (i = 0; i < NUMBER_OF_THREADS; i++)
   {
      ptr = malloc(sizeof(int));  // get rid of the race condition
      *(int*)ptr = i;
      hthread[i] = CreateThread(NULL, 0, thread, ptr, 0, NULL);
   }

   // wait for the threads to finish
   WaitForMultipleObjects(NUMBER_OF_THREADS, hthread, TRUE, INFINITE);

   printf("\nThreads stopped.\n");

   return 0;
}//main


DWORD WINAPI thread(LPVOID threadParam)
{
   int myParameter = *((int *)threadParam);
   free(threadParam);  // we no longer need the heap storage space

   printf("Hello from thread %d\n", myParameter);

   return 0;
}//thread
