/*
    This program demonstrates how a simple race condition can be
    removed.

    This program removes the race condition by removing the shared
    variable. In this case, instead of passing each thread a pointer
    to the index variable, the index variable itself is put on the
    thread's stack. So each thread's stack contains a copy of the
    index variable.

    This is not a solution that can be generalized since it is making
    use of the fact that a pointer to an int is the same size as an int.
    In general, a pointer to a data type will not be the same size as the
    data type itself.
*/
#include <windows.h>
#include <stdio.h>

int main( int argc, char *argv[])
{
   DWORD WINAPI thread(LPVOID);  //thread function prototype

   #define NUMBER_OF_THREADS 10
   HANDLE hthread[NUMBER_OF_THREADS];
   int i;

   // create threads
   printf("Starting %d threads\n", NUMBER_OF_THREADS);
   for (i = 0; i < NUMBER_OF_THREADS; i++)
   {
      // get rid of the race condition
      hthread[i] = CreateThread(NULL, 0, thread, (void*)i, 0, NULL);
   }

   // wait for the threads to finish
   WaitForMultipleObjects(NUMBER_OF_THREADS, hthread, TRUE, INFINITE);

   printf("\nThreads stopped.\n");

   return 0;
}//main


DWORD WINAPI thread(LPVOID threadParam)
{
   int myParameter = (int)threadParam;
   printf("Hello from thread %d\n", myParameter);

   return 0;
}//thread
