/*
    This program combines ideas from 6-race-example.c and
    7-race-example-fixed1.c so that in this program each
    thread has one shared variable that has a race condition
    and each thread has a "private" variable with no race condition.
*/
#include <windows.h>
#include <stdio.h>

struct vars { int var; int* ptr_var; };

// Structure to hold two thread parameters
typedef struct
{
   int  parameter;
   int* parameter_p;
}ThreadStuff_t;


int main( int argc, char *argv[])
{
   DWORD WINAPI thread(LPVOID);  //thread function prototype

   #define NUMBER_OF_THREADS 15
   HANDLE hthread[NUMBER_OF_THREADS];
   int i;

   ThreadStuff_t* ptr;

   // create threads
   printf("Starting %d threads\n", NUMBER_OF_THREADS);
   for (i = 0; i < NUMBER_OF_THREADS; i++)
   {
      ptr = (ThreadStuff_t*)malloc(sizeof(ThreadStuff_t));
      (ptr -> parameter)   = i;  // make a copy of i
      (ptr -> parameter_p) = &i; // i is a shared variable
      hthread[i] = CreateThread(NULL, 0, thread, ptr, 0, NULL);
   }

   // wait for the threads to finish
   WaitForMultipleObjects(NUMBER_OF_THREADS, hthread, TRUE, INFINITE);

   printf("\nThreads stopped.\n");

   return 0;
}//main


DWORD WINAPI thread(LPVOID threadParam)
{
   int myParameter, sharedParameter;

   // Let the scheduler reschedule these threads in a different order.
   //static int x = 0; // count the number of times this thread is instantiated
   //srand(x++);       // use that to give each thread a different seed value
   //if (rand()%2==0) Sleep(0);  // about half the threads will sleep

   myParameter = (*((ThreadStuff_t*)threadParam)).parameter;
   sharedParameter = *((*((ThreadStuff_t*)threadParam)).parameter_p);

   // alternative notation for accessing the thread parameters
   myParameter = ((ThreadStuff_t*)threadParam) -> parameter;
   sharedParameter = *(((ThreadStuff_t*)threadParam) -> parameter_p);

   free(threadParam);  // we no longer need the heap storage space

   fprintf(stderr, "Hello with myParameter = %d and sharedParameter = %d\n", myParameter, sharedParameter);

   return 0;
}//thread
