/*
    This program demonstrates how a simple "race condition" can come
    up when threads share a variable.

    The shared variable is the loop index variable i. Since each
    thread is passed a pointer to i, each thread is accessing the
    same memory location for its thread parameter. The actual value
    that any thread sees in its thread parameter depends on when the
    thread is started. That is, the actual value that a thread sees
    for its thread parameter will be the value in i when the thread
    starts, NOT the value that was in i when pthread_create() was
    called to create the thread.
*/
#include <stdio.h>    // printf()
#include "pthread.h"

int main( int argc, char *argv[])
{
   void *thread(void *vargp);  //thread function prototype

   #define NUMBER_OF_THREADS 6
   pthread_t tid[NUMBER_OF_THREADS];
   int i;

   // create threads
   printf("Starting %d threads\n", NUMBER_OF_THREADS);
   for (i = 0; i < NUMBER_OF_THREADS; i++)
      pthread_create(&(tid[i]), NULL, thread, &i);

   // wait for the threads to finish
   for (i = 0; i < NUMBER_OF_THREADS; i++)
      pthread_join(tid[i], NULL);

   printf("\nThreads stopped.\n");

   return 0;
}//main


void *thread(void *threadParam)
{
   int myParameter = *((int *)threadParam);
   printf("Hello from thread %d\n", myParameter);

   return NULL;
}//thread
