/*
    This program demonstrates how a simple race condition can be
    removed.

    This program removes the race condition by removing the shared
    variable. In this case, instead of passing each thread a pointer
    to the index variable, the thread is passed a pointer to a
    dynamically created copy of the index variable. So each thread
    gets its own dynamically allocated thread param
*/
#include <stdio.h>    // printf()
#include <stdlib.h>   // malloc(), free()
#include "pthread.h"

int main( int argc, char *argv[])
{
   void *thread(void *vargp);  //thread function prototype

   #define NUMBER_OF_THREADS 6
   pthread_t tid[NUMBER_OF_THREADS];
   int i;
   void* ptr;

   // create threads
   printf("Starting %d threads\n", NUMBER_OF_THREADS);
   for (i = 0; i < NUMBER_OF_THREADS; i++)
   {
      ptr = malloc(sizeof(int));  // get rid of the race condition
      *(int*)ptr = i;
      pthread_create(&(tid[i]), NULL, thread, ptr);
   }

   // wait for the threads to finish
   for (i = 0; i < NUMBER_OF_THREADS; i++)
      pthread_join(tid[i], NULL);

   printf("\nThreads stopped.\n");

   return 0;
}//main


void *thread(void *threadParam)
{
   int myParameter = *((int *)threadParam);
   free(threadParam);  // we no longer need the heap storage space

   printf("Hello from thread %d\n", myParameter);

   return NULL;
}//thread
