/*
    This program demonstrates how a simple race condition can be
    removed.

    This program removes the race condition by removing the shared
    variable. In this case, instead of passing each thread a pointer
    to the index variable, the index variable itself is put on the
    thread's stack. So each thread's stack contains a copy of the
    index variable.
*/
#include <stdio.h>    // printf()
#include "pthread.h"

int main( int argc, char *argv[])
{
   void *thread(void *vargp);  //thread function prototype

   #define NUMBER_OF_THREADS 6
   pthread_t tid[NUMBER_OF_THREADS];
   int i;

   // create threads
   printf("Starting %d threads\n", NUMBER_OF_THREADS);
   for (i = 0; i < NUMBER_OF_THREADS; i++)
   {
      // get rid of the race condition
      pthread_create(&(tid[i]), NULL, thread, (void*)i);
   }

   // wait for the threads to finish
   for (i = 0; i < NUMBER_OF_THREADS; i++)
      pthread_join(tid[i], NULL);

   printf("\nThreads stopped.\n");

   return 0;
}//main


void *thread(void *threadParam)
{
   int myParameter = (int)threadParam;
   printf("Hello from thread %d\n", myParameter);

   return NULL;
}//thread
